:root {
  --min-container-height: 530px;
}
#container {
  position: fixed;
  right: 40px;
  bottom: 15px;
  min-width: 700px;
  background-color: #fff;
  border-left: 1px solid #aaa;
  border-top: 1px solid #aaa;
  z-index: 105;
  min-height: var(--min-container-height);
  font-family: 'Roboto', sans-serif;
}

.bttn-txt {
  -webkit-transform: rotate(90deg);
  -moz-transform: rotate(90deg);
  -ms-transform: rotate(90deg);
  -o-transform: rotate(90deg);
  transform: rotate(90deg);
  width: 100%;
  height: 60%;
  display: inline-block;
}

.ttgt-wrapper {
  position: absolute;
  right: 0;
  bottom: 0;
  background-color: #fff;
  border-left: 1px solid #aaa;
  border-top: 1px solid #aaa;
  z-index: 105;
}

.hide {
  display: none;
}

.show {
  display: inherit;
}

.grid {
  display: flex;
  justify-content: space-between;
  border: 1px solid #aaa;
  height: 100%;
  min-height: var(--min-container-height);
}

.stats {
  float: left;
  width: 200px;
  background-color: #f2f2f2;
  box-shadow: 5px 0 5px -5px #888;
  padding: 20px;
  padding-top: 40px;
  display: flex;
  flex-direction: column;
}
.konva-grid {
  margin: 10px 0px;
  padding-right: 20px;
}

.stats-item {
  float: left;
  height: 75px;
}

.stats-title {
  text-transform: uppercase;
  margin-bottom: 4px;
  font-size: 14px;
  color: #00000073;
}

.stats-value {
  color: #828282;
  font-size: 24px;
}

.konva-body {
  flex: 1;
}

.alert {
  margin-top: 0.4rem;
  margin-left: 0.4rem;

  position: relative;
  padding: 0.75rem 1.25rem;
  border: 1px solid transparent;
  border-radius: 0.25rem;
  width: 50%;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  min-height: 20px;
}

.alert-success {
  color: #155724;
  background-color: #d4edda;
}

.alert-info {
  color: #818182;
  background-color: #fefefe;
  border-color: #d6d8db;
}

.alert-danger {
  color: #721c24;
  background-color: #f8d7da;
}

.rotate-bttn {
  vertical-align: top;
  transform: rotate(7deg);
  -ms-transform: rotate(90deg); /* IE 9 */
  -moz-transform: rotate(90deg); /* Firefox */
  -webkit-transform: rotate(90deg); /* Safari and Chrome */
  -o-transform: rotate(90deg); /* Opera */
  border-radius: 5px;
  padding: 0 10px;
  font-size: 20px;
  background-color: #f8f5e1;
}
.rotate-bttn:hover {
  box-shadow: 0px 0px 4px #fff;
}

.rotate-bttn:active {
  top: 1px;
}

.ttgt-sidebar-bttn {
  position: fixed;
  right: 0;
  bottom: 0;
  display: grid;
  z-index: 105;
  grid-template-columns: repeat(1, 1fr);
  justify-items: center;
  padding: 40px 0;
  margin: 0 auto;
}
